@@ -288,6 +288,7 @@ class PassportLogin(HTTPClient):
     def login_redir(self, info):
         self.deferred.callback((LOGIN_REDIRECT, self.headers['location'], self.authData))
 
+
 class MSNProtocolError(Exception):
     """
     This Exception is basically used for debugging
@@ -299,8 +300,21 @@ class MSNProtocolError(Exception):
     """
     pass
 
-class MSNMessage:
 
+class MSNCommandFailed(Exception):
+    """
+    The server said that the command failed.
+    """
+
+    def __init__(self, errorCode):
+        self.errorCode = errorCode
+
+    def __str__(self):
+        return ("Command failed: %s (error code %d)"
+                % (errorCodes[self.errorCode], self.errorCode))
+
+
+class MSNMessage:
     """
     I am the class used to represent an 'instant' message.
 
@@ -626,12 +640,14 @@ class MSNEventBase(LineReceiver):
         if len(cmd) != 3:
             raise MSNProtocolError, "Invalid Command, %s" % repr(cmd)
         if cmd.isdigit():
-            if self.ids.has_key(params.split()[0]):
-                self.ids[id].errback(int(cmd))
+            errorCode = int(cmd)
+            id = int(params.split()[0])
+            if id in self.ids:
+                self.ids[id][0].errback(MSNCommandFailed(errorCode))
                 del self.ids[id]
                 return
             else:       # we received an error which doesn't map to a sent command
-                self.gotError(int(cmd))
+                self.gotError(errorCode)
                 return
 
         handler = getattr(self, "handle_%s" % cmd.upper(), None)
@@ -1801,6 +1817,9 @@ class SwitchboardClient(MSNEventBase):
                     continue
             if self._checkFileInvitation(message, info) or self._checkFileInfo(message, info) or self._checkFileResponse(message, info):
                 return 0
+        elif 'text/x-clientcaps' in cTypes:
+            # do something with capabilities
+            return 0
         return 1
 
     # negotiation
@@ -2251,9 +2270,11 @@ class FileSend(LineReceiver):
         self.connected = 1
 
     def connectionLost(self, reason):
-        if self._pendingSend:
+        if self._pendingSend.active():
             self._pendingSend.cancel()
             self._pendingSend = None
+        if self.bytesSent == self.fileSize:
+            self.completed = 1
         self.connected = 0
         self.file.close()
 
@@ -2315,8 +2336,8 @@ class FileSend(LineReceiver):
         if data:
             dataSize = len(data)
             header = self.makeHeader(dataSize)
-            self.transport.write(header + data)
             self.bytesSent += dataSize
+            self.transport.write(header + data)
             self._pendingSend = reactor.callLater(0, self.sendPart)
         else:
             self._pendingSend = None
